// copyright olivier Dahan. 
// Version 1.01 Jan-2000
// mail: odahan@cybercable.fr
// site : http://perso.cybercable.fr/stargate
// You can use this library freely for personnal and commercial use.
// This is a freeware, this mean I keep my copyrights. No plublication not
// other uses without my authorization.
// Web masters : do not put this file on your site, put a link to my site please,
// so users will be sure to get the latest release.
// CD Rom makers : you can put this file on you CD if you send me free copy of
// the CD (email me for postal address).

// License : Use at your own risks. I'm not responsible of anything.
// if you don't agree these terms, you don't have the right to use this
// lib and you must suppress it from all your disks.

unit uODDskInfo;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ShellAPI;


type
  TODDriveType = (dtUnknown, dtNoDrive, dtFloppy, dtFixed, dtNetwork, dtCDROM,
                dtRAM, dtFloppy3720, dtFloppy3144, dtFloppy3288, dtFloppy3288M,
                dtFloppy5360,dtFloppy512);

  TODDriveShellInfo = record
    Icon : hIcon;     // icon as displayed by system
    Image : integer;  // icon # in the system image list
    DisplayName,      // name as in the Explorer drive combo
    TypeName : string // drive type
  end;


  TODVolumeInformation = record
    VolumeName  : string;          // volume name
    VolumeSerialNumber:dword;      // serial #
    MaximumComponentLength,        // max length of a filename
    FileSystemFlags : Cardinal;    // see below
    FileSystemName : string;       // File system name : FAT, HPFS, CDFS or NTFS ..
  end;

  // File system Flags meaning
  // Value                        Meaning
  // FS_CASE_IS_PRESERVED	  If this flag is set, the file system preserves the case of filenames when it places a name on disk.
  // FS_CASE_SENSITIVE	          If this flag is set, the file system supports case-sensitive filenames.
  // FS_UNICODE_STORED_ON_DISK	  If this flag is set, the file system supports Unicode in filenames as they  appear on disk.
  // FS_PERSISTENT_ACLS	          If this flag is set, the file system preserves and enforces access control  lists. For example, NTFS preserves and enforces ACLs, HPFS and FAT do not.
  // FS_FILE_COMPRESSION	  The file system supports file-based compression.
  // FS_VOL_IS_COMPRESSED	  The specified volume is a compressed volume; for example, a DoubleSpace volume.

  TODFileAttr = (odfaReadOnly,odfaHidden,odfaSysFile,odfaVolumeID,
                 odfaDirectory,odfaArchive);
  TODFileAttrs= Set of TODFileAttr;

  TODFileInformation = Record
   Name : string;
   DateStamp : tdatetime;
   Size : cardinal;
   exSize : Int64;
   Creation,
   LastAccess,
   Lastwrite : tdatetime;
   AltName : string;
   Attr : TODFileAttrs;
   end;

type
  TODVersionType = (vtComments, vtCompanyName, vtFileDescription, vtFileVersion,
                  vtInternalName, vtLegalCopyright, vtLegalTrademarks,
                  vtOriginalFilename, vtPrivateBuild, vtProductName,
                  vtProductVersion, vtSpecialBuild, vtSignature, vtStructVersion,
                  vtFileVersionInt, vtProductVersionInt, vtFileOS, vtFileType,
                  vtFileSubType, vtFileDate, vtFileFlags);

  ODPVS_FIXEDFILEINFO = ^VS_FIXEDFILEINFO;

  TODVersionReader = class
  private
    { Storage areas }
    FFullFilename: string;                  { Full Drive/Path/and filename }
    FVersionExists: boolean;                { False if EXE/DLL has no Version info }
    FVersionBuffer: Pointer;                { Buffer containing Version info }
    FVersionSize: LongInt;                  { Buffer Size }
    FVersionHandle : Cardinal;               { Version functions Handle }
    FFixedInfoBuffer: ODPVS_FIXEDFILEINFO;    { Fixed File Information block }
    FLanguagePath: array[0..79] of char;    { Language Path }
    FLanguageName: string;                  { Language Name }
    { Methods }
    function GetVersionInfo(Filename: String): boolean;

    procedure ClearBuffer;
    procedure SetFullFileName(Value: String);
    function GetAllVersionInfo : string;

  public
    constructor Create;
    constructor CreateWithFile(FullFile: String);
    destructor Destroy; override;

    function Read(VersionType: TODVersionType): string;
    procedure Clear;

    property Filename: string read FFullFilename write SetFullFilename;
    property Language: string read FLanguageName;
    property VersionExists: boolean read FVersionExists;
    property AllVersionInfo : string read getAllVersionInfo;
  end;



Const
  kODDriveType : array[TODDriveType] of string =
                 ('Unkown','No Drive','Floppy','Fixed','NetWork','CD-ROM',
                  'RAM','Floppy 3 720k', 'Floppy 3 1.44M','Floppy 3 2.88M',
                  'Floppy 3 >2.88M', 'Floppy 5 360k','Floppy 5 1.2M');
  kODFileAttr : array[TODFileAttr,Boolean] of string =
                (('R','ReadOnly'),('H','Hidden'),('S','System'),
                 ('V','Volume ID'),('D','Directory'),('A','Archive'));


Var
 vODLargeImages : TImageList;  // large system icons
 vODLittleImages : TImageList; // little system icons

// Returns the list of drives known by the OS
procedure GetLogicalDriveList (List : TStrings);
// Returns the logical type of Drive
function GetDriveType (Drive : Char) : TODDriveType;
// Returns logical drive type as String
function GetDriveTypeStr (Drive : Char) : String;
// Returns true subtype for Floppy
function FloppyDriveSize (Drive : char) : TODDriveType;
// Returns Shell info about drive
procedure GetDriveShellInfo (Drive : Char; var Info : TODDriveShellInfo);
// Returns volume information
function GetVolumeInformation (D : char; var V : TODVolumeInformation) : boolean;
// Formats a disk serial # as usualy displayed by the OS
function GetSerialStr(Serial:Integer):String;
// Returns System flags (GetVolumeinformation) as a string list
Procedure GetSysFlagsStr(sysFlags:Integer;Rslt:TStrings);
// Returns last disk error as a string
function DiskErrorMessage (Drive : Char) : string;
// Dialog returning True if user want to retry
function DiskRetry (Drive : Char) : boolean;
// Returns drive space information (int64 -> > 2Go are ok)
Function GetDiskSpace(Drive : char;
                      var FreeBytesAvailableToCaller,
                          TotalNumberOfBytes,
                          TotalNumberOfFreeBytes : TLargeInteger) : boolean;
// Returns the Network volume name
function GetNetworkVolumeName (D : char): string;
// Returns the volume name
function GetVolumeName (D : char) : string;

// *** FILE INFO ***

// File shell info - icon is created and must be freed by caller !
procedure getshellinfo(const a:tfilename;var name,typ:string;var icon:ticon;var attr,image:integer);
// Returns file information
Function GetFileInfo(const FileName: String; var Info:TSearchRec):boolean;
// returns short full name (with path)
function ExtractShortPathName(const FileName: string): string;
// Returns a TimeDate for a File date time (TsearchRec extended info)
function GetDateTimeForFileDate(t:tfiletime):TDateTime;
// Returns file info in usable format (replaces getFileInfo and getDateTimeForFileDate)
function GetODFileInfo(const Filename:string;var info:TODFileInformation):boolean;
// Returns file attributes as string (long or short)
function FileAttrStr(attr:TODFileAttrs;Long:boolean=false;Multiline:boolean=false):string;

implementation

const
  kInfoNum = 10;
  kInfoStr : array [1..kInfoNum] of String =
    ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName',
     'LegalCopyright', 'LegalTradeMarks', 'OriginalFilename',
     'ProductName', 'ProductVersion', 'Comments');
  kInfoStrTxt : array [1..kInfoNum] of String =
    ('Company Name', 'Description', 'File Version', 'Internal Name',
     'Copyright', 'Trade Marks', 'Original file name',
     'Product Name', 'Product Version', 'Comments');
  MaxStrLength = 50;                        // Maximum needed string length / version info


// --- RealizeLength ---
procedure RealizeLength(var S: string);
begin
  SetLength (S, StrLen (PChar(S)))
end;

// --- GetLogicalDriveList ---
procedure GetLogicalDriveList (List : TStrings);
	var
	  Num  : integer;
	  Bits : set of 0..25;
	begin
	  List.Clear;
	  integer (Bits) := Windows.GetLogicalDrives;
	  for Num := 0 to 25 do
	    if Num in Bits then
	      List.Add (Char (Num + Ord('A')) + ':\')
	end;

// --- GetDriveType ---
function GetDriveType (Drive : Char) : TODDriveType;
begin
  Result := TODDriveType (Windows.GetDriveType(PChar(Drive + ':\')));
  if Result = dtFloppy then
   Result := FloppyDriveSize(Drive);
end;

// --- GetDriveTypeStr ---
function GetDriveTypeStr (Drive : Char) : String;
begin
 Result := kODDriveType[GetDriveType(Drive)];
end;

// --- FloppyDriveSize ---
function FloppyDriveSize (Drive : char) : TODDriveType;
type
  PDIOC_REG = ^TDIOC_Registers;
  TDIOC_Registers = record
    Reg_EBX, Reg_EDX, Reg_ECX, Reg_EAX,
    Reg_EDI, Reg_ESI, Reg_Flags : DWORD
  end;
const
  VWIN32_DIOC_DOS_INT13 = 4;
var
  H : THandle;
  R : TDIOC_Registers;
  C : DWORD;
begin
  Result := dtFloppy;
  H := CreateFile ('\\.\VWIN32', 0, 0, nil, 0, 0, 0);

if H <> INVALID_HANDLE_VALUE then
  try
    R.Reg_EAX := $800;
    R.Reg_EDX := ord (upcase(Drive)) - Ord('A');
    if DeviceIOControl (H, VWIN32_DIOC_DOS_INT13, @R,
         SizeOf (R), @R, SizeOf (R), C, nil) and
         (R.Reg_Flags and 1 = 0) then
      case R.Reg_EBX and $FF of
       1: result := dtFloppy5360;
       2: result := dtFloppy512;
       3: result := dtFloppy3720;
       4: result := dtFloppy3144;
       5: result := dtFloppy3288;
       else result := dtFloppy3288M;
      end;
finally
    CloseHandle (H)
  end
end;

// --- GetDriveShellInfo ---
procedure GetDriveShellInfo (Drive : Char; var Info : TODDriveShellInfo);
var
  SHFileInfo : TSHFileInfo;
begin
  ShGetFileInfo (PChar (Drive + ':\'), 0, SHFileInfo, SizeOf (TSHFileInfo),
     SHGFI_TYPENAME or SHGFI_DISPLAYNAME or SHGFI_SYSICONINDEX or SHGFI_ICON);
with Info do
  begin
    Icon  := SHFileInfo.hIcon;
    Image := SHFileInfo.iIcon;
    DisplayName := SHFileInfo.szDisplayName;
    TypeName := SHFileInfo.szTypeName
  end
end;


// --- GetVolumeInformation ---
function GetVolumeInformation (D : char; var V : TODVolumeInformation) : boolean;
var
  O : integer;
begin
  O := SetErrorMode (SEM_FAILCRITICALERRORS);
  try
    with V do
     begin
      SetLength (VolumeName, MAX_PATH);
      SetLength (FileSystemName, MAX_PATH);
      VolumeSerialNumber := 0;
      MaximumComponentLength := 0;
      FileSystemFlags := 0;
      Result := Windows.GetVolumeInformation (PChar (D+':\'), PChar (VolumeName), MAX_PATH,
        @VolumeSerialNumber, MaximumComponentLength, FileSystemFlags,
        PChar (FileSystemName), MAX_PATH);
      RealizeLength (VolumeName);
      RealizeLength (FileSystemName)
     end
  finally
    SetErrorMode (O)
  end
end;


// --- GetSerialStr ---
function GetSerialStr(Serial:Integer):String;
begin
result := Format ('%4x-%4x', [longrec(Serial).hi, longrec(Serial).lo]);
while pos(' ',result)>0 do result[pos(' ',result)]:='0';
end;

// --- GetSysFlagsStr ---
Procedure GetSysFlagsStr(sysFlags:Integer;Rslt:TStrings);
begin
 Rslt.Clear;
 if SysFlags and FS_CASE_IS_PRESERVED <> 0 then
        rslt.Add('Case is preserved in file name');
 if SysFlags and FS_CASE_SENSITIVE <> 0 then
        rslt.add('Case sensitive file name');
 if SysFlags and FS_UNICODE_STORED_ON_DISK <> 0 then
        rslt.add('Unicode in file name allowed');
 if SysFlags and FS_PERSISTENT_ACLS <> 0 then
        rslt.Add('Persistent Access Control Lists');
 if SysFlags and FS_VOL_IS_COMPRESSED <> 0 then
        rslt.add('Volume is compressed');
 if SysFlags and FS_FILE_COMPRESSION <> 0 then
        rslt.add('File compression supported');
end;

// --- DiscErrorMessage ---
function DiskErrorMessage (Drive : Char) : string;
begin
  Result := Format ('%s:\ is not accessible.'#13#10+'%s',
                     [uppercase(Drive), SysErrorMessage (GetLastError)]);
end;

// --- DiskRetry ---
function DiskRetry (Drive : Char) : boolean;
begin
  Result := Application.MessageBox (
              PChar (DiskErrorMessage (Drive)),
              PChar (Application.Title),
              mb_RetryCancel or mb_IconError) = idRetry;
end;


// --- GetDiskSpace ---
Function GetDiskSpace(Drive : char;
                      var FreeBytesAvailableToCaller,
                          TotalNumberOfBytes,
                          TotalNumberOfFreeBytes : TLargeInteger) : boolean;
var pc:array[0..10] of char;
begin
StrpCopy(pc,Drive+':');
Result := GetDiskFreeSpaceEx(pc,FreeBytesAvailableToCaller,
                                       TotalNumberOfBytes,
                                      @TotalNumberOfFreeBytes);
end;


// --- GetVolumeName ---
function GetVolumeName (D : char) : string;
var
  T : TODVolumeInformation;
begin
  if GetVolumeInformation (D, T) then
    Result := T.VolumeName
  else
    Result := ''
end;

// --- GetNetworkVolumeName ---
function GetNetworkVolumeName (D : char): string;
var
  L : Cardinal;
begin
  L := MAX_PATH;
  SetLength (Result, L);

  if WNetGetConnection (PChar(D+':'#0), PChar(Result), L) = NO_ERROR then
    RealizeLength (Result)
  else
    Result := GetVolumeName(D)
end;



// --- GetImageList --- internal use . not published
function GetImageList (Option : integer) : TImageList;
var
  SHFileInfo : TSHFileInfo;
begin
  Result := TImageList.Create (nil);
  with Result do
  begin
    ShareImages := true;

    Handle := ShGetFileInfo ('*.*', 0, SHFileInfo, SizeOf (TSHFileInfo),
                   Option or SHGFI_SYSICONINDEX)
  end
end;


// --- getshellinfo ---
procedure getshellinfo(const a:tfilename;var name,typ:string;var icon:ticon;var attr,image:integer);
var
  info : tshfileinfo;
begin
     // get shell-info about the specified file (in "a")
     {parameters :
          name : receives the explorer-style filename (e.g. without extension)
          typ  : receives the explorer-style typename of the file (e.g. "anwendung" for an exe in german win95)
          icon : will content the icon which is shown in the explorer for that file (this is slower than extracticon !!!!)
          attr : gets the explorer-attributes of the file, these are (ripped from win32-help):
               sfgao_cancopy the specified file objects or folders can be copied (same value as the dropeffect_copy flag).
               sfgao_candelete the specified file objects or folders can be deleted.
               sfgao_canlink it is possible to create shortcuts for the specified file objects or folders (same value as the dropeffect_link flag).
               sfgao_canmove the specified file objects or folders can be moved (same value as the dropeffect_move flag).
               sfgao_canrename the specified file objects or folders can be renamed.
               sfgao_capabilitymask mask for the capability flags.
               sfgao_droptarget the specified file objects or folders are drop targets.
               sfgao_haspropsheet the specified file objects or folders have property sheets.

               a file object's display attributes may include zero or more of the following values:

               sfgao_displayattrmask mask for the display attributes.
               sfgao_ghosted the specified file objects or folders should be displayed using a ghosted icon.
               sfgao_link the specified file objects are shortcuts.
               sfgao_readonly the specified file objects or folders are read-only.
               sfgao_share the specified folders are shared.

               a file object's contents flags may include zero or more of the following values:

               sfgao_contentsmask mask for the contents attributes.
               sfgao_hassubfolder the specified folders have subfolders (and are, therefore, expandable in the left pane of windows explorer).

               a file object may have zero or more of the following miscellaneous attributes:

               sfgao_filesystem the specified folders or file objects are part of the file system (that is, they are files, directories, or root directories).
               sfgao_filesysancestor the specified folders contain one or more file system folders.
               sfgao_folder the specified items are folders.
               sfgao_removable the specified file objects or folders are on removable media.
               sfgao_validate validate cached information.}


     fillchar(info,sizeof(tshfileinfo),0);
     shgetfileinfo(pchar(a),0,info,sizeof(info),shgfi_displayname or shgfi_typename or
     shgfi_icon or shgfi_attributes);
     with info do begin
          name := szdisplayname;
          typ  := sztypename;
          icon := Ticon.Create;
          image:= info.iIcon;
          icon.handle := hicon;
          attr        := dwattributes;
     end;
end;

// --- GetFileInfo ---
Function GetFileInfo(const FileName: String; var Info:TSearchRec):boolean;
BEGIN;
 Result:= FindFirst(Filename,faAnyFile,info)=0;
 FindClose(info);
END;

// --- ExtractShortPathName ---
function ExtractShortPathName(const FileName: string): string;
var
Buffer: array[0..MAX_PATH] of Char;
begin
SetString(Result, Buffer,
GetShortPathName(PChar(FileName), Buffer, SizeOf(Buffer)));
end;

// --- GetDateTimeForFileDate ---
function GetDateTimeForFileDate(t:tfiletime):TDateTime;
var i:integer; localfiletime:tfiletime;
begin
 if FileTimeToLocalFileTime(t, LocalFileTime) then
  begin
  if FileTimeToDosDateTime(LocalFileTime, LongRec(i).Hi,LongRec(i).Lo) then
   try
    Result := FileDateToDateTime(i);
   except
    Result := 0;
   end else Result:=0;
  end else result:=0;
end;

// --- GetODFileInfo ---
function GetODFileInfo(const Filename:string;var info:TODFileInformation):boolean;
type li = record low,high:cardinal; end;
var inf : tsearchrec; var x:Int64; vli:li absolute x;
begin
 result := GetFileInfo(filename,inf);
 if not result then exit;
 info.Name := inf.Name;
 info.DateStamp := FileDateToDateTime(inf.Time);
 info.Size := inf.Size;
 Info.Creation   := GetDateTimeForFileDate(inf.FindData.ftCreationTime);
 info.LastAccess := GetDateTimeForFileDate(inf.FindData.ftLastAccessTime);
 info.Lastwrite  := GetDateTimeForFileDate(inf.FindData.ftLastWriteTime);
 info.AltName    := trim(inf.FindData.cAlternateFileName);
 if Info.Altname = '' then Info.Altname := Info.Name;
 vli.low         := inf.FindData.nFileSizeLow;
 vli.high        := inf.FindData.nFileSizeHigh;
 info.exSize     := x;
 Info.Attr       := [];
 if (inf.Attr and faReadOnly) <> 0 then Info.attr := info.Attr+[odfaReadOnly];
 if (inf.Attr and faHidden) <> 0 then Info.attr := info.Attr+[odfaHidden];
 if (inf.Attr and faSysFile) <> 0 then Info.attr := info.Attr+[odfaSysFile];
 if (inf.Attr and faVolumeID) <> 0 then Info.attr := info.Attr+[odfaVolumeID];
 if (inf.Attr and faDirectory) <> 0 then Info.attr := info.Attr+[odfaDirectory];
 if (inf.Attr and faArchive) <> 0 then Info.attr := info.Attr+[odfaArchive];
end;

// --- FileAttrStr ---
function FileAttrStr(attr:TODFileAttrs;Long:boolean=false;Multiline:boolean=false):string;
var i:TODFileAttr;
begin
 result:='';
 for i:=Low(TODFileAttr) to High(TODFileAttr) do
  if i in Attr then
   begin
    if result<>'' then
     begin
      if long then
       begin
        if multiline then result:=result+#13#10
                     else result:=result+', ';
       end
         else result:=result+' ';
     end;
    Result:=Result+kODFileAttr[i,long];
   end else if not long then result:=result+'-';
end;


// --- TODVersionReader.Create ---
constructor TODVersionReader.Create;
begin
  inherited Create;                         { Create underlying object }
  ClearBuffer;                              { Reset all member variables }
end;


// --- TODVersionReader.CreateWithFile ---
constructor TODVersionReader.CreateWithFile(FullFile: String);
begin
  inherited Create;                         { Create underlying object }
  ClearBuffer;                              { Reset all member variables }
  SetFullFilename(FullFile);                { Find and Read in File Version }
end;

// --- TODVersionReader.Destroy ---
destructor TODVersionReader.Destroy;
begin
  ClearBuffer;                              { Release version resource memory }
  inherited Destroy;                        { Destroy the object }
end;

// --- TODVersionReader.ClearBuffer ---
procedure TODVersionReader.ClearBuffer;
begin
  if Assigned(FVersionBuffer) then
    FreeMem(FVersionBuffer, FVersionSize + 1);  { Release Version resource memory }
  FVersionBuffer := nil;
  FFixedInfoBuffer := nil;
  FFullFilename := '';
  FVersionExists := False;
  FVersionSize := 0;
  FVersionHandle := 0;
  FLanguageName := '';
  FillChar(FLanguagePath, SizeOf(FLanguagePath), 0);
end;

// --- TODVersionReader.Clear ---
procedure TODVersionReader.Clear;
begin
  ClearBuffer;                              { Reset all member variables }
end;

// --- TODVersionReader.SetFullFilename ---
procedure TODVersionReader.SetFullFilename(Value: string);
begin
  if Value <> FFullFileName then            { If a new file name }
  begin
    if Value = '' then
      ClearBuffer                           { Release old information }
    else
      FVersionExists :=
        GetVersionInfo(Value);              { Load up new Version info }
    FFullFilename := Value;                 { Set internal file name }
  end;
end;

// --- TODVersionReader.GetVersionInfo ---
function TODVersionReader.GetVersionInfo(Filename: String): boolean;
var
  PCharFile: PChar;
  PLangName: PChar;
  Len: Cardinal;
  LangCharSetIDArray: array[1..2] of word;  { Language ID }
  TranslationInfoPointer: Pointer;          { Language set pointer }
  s:string;
begin
  Result := False;                          { Assume failure }
  ClearBuffer;                              { Reset all member variables }
  PLangName := StrAlloc(MaxStrLength);      { Allocate space for Language Name }
  PCharFile :=
    StrAlloc(Length(Filename) + 1);         { Allocate Filename string }
  StrPCopy(PCharFile, Filename);            { Copy Pascal string to PCHAR }

  try
    FVersionSize := GetFileVersionInfoSize(
      PCharFile, FVersionHandle);           { Attempt to read Version Info }
    if FVersionSize > 0 then                { If we got one }
    begin
      GetMem(FVersionBuffer,
        FVersionSize + 1);                  { Allocate Memory for it }
      FillChar(FVersionBuffer^,
        FVersionSize + 1, 0);               { Zero memory allocated }
      try
        if not GetFileVersionInfo(PCharFile,
          FVersionHandle, FVersionSize,
          FVersionBuffer) then              { If we can't read Version info }
          ClearBuffer;                      { Clear internals out }

        if VerQueryValue(FVersionBuffer,
          '\VarFileInfo\Translation',
          TranslationInfoPointer, Len) then { Get Language Translation code }
        begin
          LangCharSetIDArray[1] := LoWord(
            PLongint(TranslationInfoPointer)^);
          LangCharSetIDArray[2] := HiWord(
            PLongint(TranslationInfoPointer)^);{ Set Language LongInt codes }

          (*wvSprintf(FLanguagePath, '\StringFileInfo\%04x%04x\',
            pchar(LangCharSetIDArray));            { Slot codes into Main Path String }*)

          s:=Format('%04x%04x',[langCharSetIDArray[1],LangCharSetIDArray[2]]);
          while pos(' ',s)>0 do s[pos(' ',s)]:='0';
          StrPCopy(FlanguagePath, ('\StringFileInfo\'+s+'\') );


          VerLanguageName(LangCharSetIDArray[1],
            PLangName, MaxStrLength);       { Retrieve Language desc text }
          FLanguageName := StrPas(PLangName);{ Copy to Pascal String }

          if VerQueryValue(FVersionBuffer, '\',
            pointer(FFixedInfoBuffer), Len) then
            Result := True;                 { Get Pointer to Fixed File info }

        end else
          raise Exception.Create('Error retrieving Language information');
      except
        ClearBuffer;
        raise;
      end;
    end;
  finally
    StrDispose(PLangName);                  { Release PCHAR strings memory }
    StrDispose(PCharFile);
  end;
end;


Function TODVersionReader.GetAllVersionInfo : string;
 function bld(title:string;id:TODVersionType;crlf:boolean=true):string;
 begin
  result := trim(Read(id));
  if result = '' then exit;
  while length(title)<24 do title:=title+#32;
  title:=copy(title,1,24);
  result:=title+': '+result;
  if crlf then result:=result+#13#10;
 end;
begin
result := bld('Product Name',vtProductName)+
          bld('Company',vtCompanyName)+
          bld('Product Version',vtProductVersion)+
          bld('Description',vtFileDescription)+
          bld('Legal Copyright',vtLegalCopyright)+
          bld('Legal Trade Marks',vtLegalTradeMarks)+
          bld('File Version',vtFileVersion)+
          bld('Comments',vtComments)+
          bld('Internal Name',vtInternalName)+
          bld('Original File Name',vtOriginalFileName)+
          bld('Private Build',vtPrivateBuild)+
          bld('Special Build',vtSpecialBuild)+
          bld('Signature',vtSignature)+
          bld('Struct Version',vtStructVersion)+
          bld('Internal File Version',vtFileVersionInt)+
          bld('Internal Product Version',vtProductVersionInt)+
          bld('File OS',vtFileOS)+
          bld('File Type',vtFileType)+
          bld('File Sub Type',vtFileSubType)+
          bld('File Date',vtFileDate)+
          bld('File Flags',vtFileFlags,false);
end;

// --- TODVersionReader.Read ---
function TODVersionReader.Read(VersionType: TODVersionType): string;

  (*
  **  Reads the relevant Member information given the path string to it
  **  (SearchStr) and the Member information required (StrName)
  *)
  function ReadStrFileInfo(const StrName: String): string;
  var
    SearchStr: PChar;
    VersPChar: Pointer;
    VerSelection: PChar;
    Len: Cardinal;
  begin
    Result := '';                           { Assume no return string }
    SearchStr := StrAlloc(MaxStrLength);    { Allocate Memory for Search Path }
    FillChar(SearchStr^, SizeOf(SearchStr), 0);
    VerSelection := StrAlloc(MaxStrLength); { Allocate space Member Text }
    try
      StrPCopy(VerSelection, StrName);      { Copy Member Descr to PCHAR }
      StrCopy(SearchStr, FLanguagePath);    { Copy base path to PCHAR }
      StrCat(SearchStr, VerSelection);      { Add member Desc text to base path }
      if VerQueryValue(FVersionBuffer,
        SearchStr, VersPChar, Len) then     { If Member information found }
        Result := StrPas(VersPChar);        { Return as a string }
    finally
      StrDispose(VerSelection);             { Release PCHAR memory }
      StrDispose(SearchStr);
    end
  end;


 function MakeFullVersion(MS, LS: Longint): string;
  begin
    Result := IntToStr((MS and $FFFF0000) shr $10) + '.' +
              IntToStr( MS and $FFFF) + '.' +
              IntToStr((LS and $FFFF0000) shr $10) + '.' +
              IntToStr( LS and $FFFF);
  end;

var
  VersPCharPath: PChar;
  TmpLong: LongInt;
begin
  Result := '';                             { Assume nothing }
  if FVersionExists then
  begin
    try
      case VersionType of
        vtComments:
          Result := ReadStrFileInfo('Comments');
        vtCompanyName:
          Result := ReadStrFileInfo('CompanyName');
        vtFileDescription:
          Result := ReadStrFileInfo('FileDescription');
        vtFileVersion:
          Result := ReadStrFileInfo('FileVersion');
        vtInternalName:
          Result := ReadStrFileInfo('InternalName');
        vtLegalCopyright:
          Result := ReadStrFileInfo('LegalCopyright');
        vtLegalTrademarks:
          Result := ReadStrFileInfo('LegalTrademarks');
        vtOriginalFilename:
          Result := ReadStrFileInfo('OriginalFilename');
        vtPrivateBuild:
          Result := ReadStrFileInfo('PrivateBuild');
        vtProductName:
          Result := ReadStrFileInfo('ProductName');
        vtProductVersion:
          Result := ReadStrFileInfo('ProductVersion');
        vtSpecialBuild:
          Result := ReadStrFileInfo('SpecialBuild');
        vtSignature:
          Result := Format('%8x', [FFixedInfoBuffer^.dwSignature]);
        vtStructVersion:
          Result := IntToStr((FFixedInfoBuffer^.dwStrucVersion and $FFFF0000)
                    shr $10) + '.' +
                    IntToStr( FFixedInfoBuffer^.dwStrucVersion and $FFFF);
        vtFileVersionInt:
          Result := MakeFullVersion(FFixedInfoBuffer^.dwFileVersionMS,
                                    FFixedInfoBuffer^.dwFileVersionLS);
        vtProductVersionInt:
          Result := MakeFullVersion(FFixedInfoBuffer^.dwProductVersionMS,
                                    FFixedInfoBuffer^.dwProductVersionLS);
        vtFileOS:
          Result := IntToStr(FFixedInfoBuffer^.dwFileOS);
        vtFileType:
          Result := IntToStr(FFixedInfoBuffer^.dwFileType);
        vtFileSubType:
          Result := IntToStr(FFixedInfoBuffer^.dwFileSubType);
        vtFileDate:
          Result := IntToStr(FFixedInfoBuffer^.dwFileDateMS) + '.' +
                    IntToStr(FFixedInfoBuffer^.dwFileDateLS);
        vtFileFlags:
          Result := IntToStr(FFixedInfoBuffer^.dwFileFlags and
                             FFixedInfoBuffer^.dwFileFlagsMask);
      end;
    except
      raise Exception.Create('Error retrieving Version information');
    end;
  end;
end;





initialization
vODLargeImages := GetImageList (SHGFI_LARGEICON);
vODLittleImages := GetImageList (SHGFI_SMALLICON);
finalization
vODLargeImages.free;
vODLittleImages.free;
end.
